/*
 * simple divider applet
 * (C) 2003 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 *
 */

#include "divider.h"
#include "pixbuf_ops.h"

/*
 *-------------------------------------
 * the style stuff
 *-------------------------------------
 */

void style_backing_sync(AppData *ad)
{
	PanelAppletBackgroundType type;
	GdkColor color;
	GdkPixmap *pixmap;
	gint w, h;

	if (!ad->display) return;
	if (!ad->pixbuf || !ad->overlay) return;

	w = gdk_pixbuf_get_width(ad->pixbuf);
	h = gdk_pixbuf_get_height(ad->pixbuf);

	type = panel_applet_get_background(PANEL_APPLET(ad->applet), &color, &pixmap);

	if (type == PANEL_COLOR_BACKGROUND || type == PANEL_NO_BACKGROUND)
		{
		gint r, g, b;

		if (type == PANEL_COLOR_BACKGROUND)
			{
			r = color.red >> 8;
			g = color.green >> 8;
			b = color.blue >> 8;

			gtk_widget_modify_bg(GTK_WIDGET(ad->applet), GTK_STATE_NORMAL, &color);
			}
		else
			{
			GtkStyle *style;
			GtkRcStyle *rc_style;
			GdkColor *bg;

			rc_style = gtk_rc_style_new();
			gtk_widget_modify_style(GTK_WIDGET(ad->applet), rc_style);
			gtk_rc_style_unref(rc_style);

			style = gtk_widget_get_style(GTK_WIDGET(ad->applet));
			bg = &style->bg[GTK_STATE_NORMAL];
			r = bg->red >> 8;
			g = bg->green >> 8;
			b = bg->blue >> 8;
			}

		pixbuf_draw_rect_fill(ad->pixbuf, 0, 0, w, h, r, g, b, 255);
		}
	else if (type == PANEL_PIXMAP_BACKGROUND)
		{
		gint x, y;
		GdkColormap *cmap;

		cmap = gdk_window_get_colormap(GTK_WIDGET(ad->applet)->window);
		x = 0;
		y = 0;
		gdk_pixbuf_get_from_drawable(ad->pixbuf, pixmap, cmap, x, y, 0, 0, w, h);
		}
	else
		{
		g_print("Unknown panel background type\n");

		pixbuf_draw_rect_fill(ad->pixbuf, 0, 0, w, h, 255, 0, 0, 255);
		}

	if (ad->shift)
		{
		GdkPixbuf *pb;

		/* we have to color shift on a copy */
		pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, gdk_pixbuf_get_has_alpha(ad->overlay), 8, w, h);
		pixbuf_copy_area(ad->overlay, 0, 0, pb, 0, 0, w, h, TRUE);

		/* now do the color shift */
		pixbuf_draw_rect_fill(pb, 0, 0, w, h,
				      ad->shift_r, ad->shift_g, ad->shift_b, ad->shift_a);

		pixbuf_copy_area_alpha(pb, 0, 0,
				       ad->pixbuf, 0, 0,
				       w, h, ad->visibility);
		gdk_pixbuf_unref(pb);
		}
	else
		{
		pixbuf_copy_area_alpha(ad->overlay, 0, 0,
				       ad->pixbuf, 0, 0,
				       w, h, ad->visibility);
		}

	if (pixmap) gdk_pixmap_unref(pixmap);

	gtk_image_set_from_pixbuf(GTK_IMAGE(ad->display), ad->pixbuf);

	gtk_widget_set_size_request(ad->display, w, h);
	/* add a 1 pixel border so the panel has a place to draw the focus outline
	 * without cropping the image
	 */
	gtk_widget_set_size_request(GTK_WIDGET(ad->applet), w + 2, h + 2);
}

static gint style_load(AppData *ad, gchar *path)
{
	GdkPixbuf *pb;
	gint sw, sh;
	gint w, h;
	float scale;

	if (!g_file_exists(path)) return FALSE;

	pb = gdk_pixbuf_new_from_file(path, NULL);

	if (!pb) return FALSE;

	if (ad->overlay) gdk_pixbuf_unref(ad->overlay);

	sw = gdk_pixbuf_get_width(pb);
	sh = gdk_pixbuf_get_height(pb);

	scale = (float)(ad->size - 2) / sh;

	if (ad->orient == PANEL_APPLET_ORIENT_LEFT || ad->orient == PANEL_APPLET_ORIENT_RIGHT)
		{
		GdkPixbuf *tmp;

		tmp = pb;
		pb = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, sh, sw);

		pixbuf_copy_area_rotate_90(tmp, 0, 0, sw, sh, pb, 0, 0, TRUE);

		gdk_pixbuf_unref(tmp);

		w = (gint)((float)sh * scale);
		h = (gint)((float)sw * scale);
		}
	else
		{
		w = (gint)((float)sw * scale);
		h = (gint)((float)sh * scale);
		}

	if (w < 1) w = 1;
	if (h < 1) h = 1;

	ad->overlay = gdk_pixbuf_scale_simple(pb, w, h, GDK_INTERP_BILINEAR);

	if (ad->pixbuf) gdk_pixbuf_unref(ad->pixbuf);
	ad->pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8, w, h);

	gdk_pixbuf_unref(pb);

	style_backing_sync(ad);

	return TRUE;
}

static void style_reload(AppData *ad)
{
	gchar *path;

	if (!ad->style_filename)
		{
		path = g_concat_dir_and_file(DIVIDER_THEME_PATH, DEFAULT_STYLE);
		if (!style_load(ad, path))
			{
			printf("divider_applet unable to load default style!\n");
			}
		g_free(path);
		}
	else
		{
		path = g_concat_dir_and_file(DIVIDER_THEME_PATH, ad->style_filename);
		if (!style_load(ad, path))
			{
			g_free(path);
			path = g_concat_dir_and_file(DIVIDER_THEME_PATH, DEFAULT_STYLE);
			if (!style_load(ad, path))
				{
				printf("divider_applet unable to load style, even default!\n");
				}
			}
		g_free(path);
		}
}

void style_set(AppData *ad, const gchar *name)
{
	gchar *path;

	if (name)
		path = g_concat_dir_and_file(DIVIDER_THEME_PATH, name);
	else
		path = g_concat_dir_and_file(DIVIDER_THEME_PATH, DEFAULT_STYLE);

	if (style_load(ad, path))
		{
		g_free(ad->style_filename);
		if (name)
			ad->style_filename = g_strdup(name);
		else
			ad->style_filename = NULL;
		}
	else
		{
		printf("divider_applet: unable to load style: %s\n", path);
		}

	g_free(path);
}


/*
 *-------------------------------------
 * the main stuff
 *-------------------------------------
 */

static void about_cb (PanelApplet *widget, gpointer data)
{
	static GtkWidget *about = NULL;
	GtkWidget *href;

	const gchar *authors[] = {
		"John Ellis <johne@bellatlantic.net>",
		NULL
		};
	const gchar *documenters[] = {
		NULL
		};
	const gchar *translators = _("translators");

	if (about != NULL)
		{
		gdk_window_show(about->window);
		gdk_window_raise(about->window);
		return;
		}

	about = gnome_about_new ( _("Divider Applet"), VERSION,
			"(C) 2003 John Ellis",
			_("Simple divider applet"),
			authors, documenters,
			strcmp (translators, "translators") != 0 ? translators : NULL,
			NULL);

	href = gnome_href_new("http://gqapplets.sourceforge.net", _("Divider home page"));
	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(about)->vbox), href, FALSE, FALSE, 0);
	gtk_widget_show(href);

	gtk_window_set_wmclass(GTK_WINDOW(about), "divider-applet", "Divider Applet");

	g_signal_connect(G_OBJECT(about), "destroy",
			 G_CALLBACK(gtk_widget_destroyed), &about );
	gtk_widget_show (about);
}

/* clean up function to free all the applet's memory and stop timers */
static void destroy_applet(GtkWidget *widget, gpointer data)
{
	AppData *ad = data;

	if (ad->propwindow)
		{
		g_signal_handlers_disconnect_matched(G_OBJECT(ad->propwindow), G_SIGNAL_MATCH_DATA,
						     0, 0, NULL, NULL, ad);
		gtk_widget_destroy(ad->propwindow);
		ad->propwindow = NULL;
		}

	if (ad->pixbuf) gdk_pixbuf_unref(ad->pixbuf);
	if (ad->overlay) gdk_pixbuf_unref(ad->overlay);

	g_free(ad->style_filename);
	g_free(ad);
}

static void applet_change_orient(GtkWidget *w, PanelAppletOrient o, gpointer data)
{
	AppData *ad = data;
	ad->orient = o;

	style_reload(ad);
}

static void applet_change_size(GtkWidget *w, int size, gpointer data)
{
	AppData *ad = data;

	ad->size = size;	

	style_reload(ad);
}

static void applet_do_draw(GtkWidget *w, PanelAppletBackgroundType type, GdkColor *color, GdkPixmap *pixmap, gpointer data)
{
	AppData *ad = data;

	style_backing_sync(ad);
}

static const BonoboUIVerb applet_menu_verbs[] = {
	BONOBO_UI_UNSAFE_VERB("Properties", property_show),
	BONOBO_UI_UNSAFE_VERB("About", about_cb),
	BONOBO_UI_VERB_END
};

static gint applet_widget_new(PanelApplet *applet)
{
        AppData *ad;

	gnome_window_icon_set_default_from_file(GNOME_ICONDIR"/divider-applet.png");

	panel_applet_add_preferences(applet, "/schemas/apps/divider-applet/prefs", NULL);

        ad = g_new0(AppData, 1);

	ad->applet = applet;
	ad->propwindow = NULL;
	ad->orient = panel_applet_get_orient(applet);
	ad->size = panel_applet_get_size(applet);
	ad->visibility = 255;

	ad->style_filename = NULL;

	property_load(ad);
	ad->display = gtk_image_new();
	g_signal_connect(G_OBJECT(ad->display), "destroy",
			 G_CALLBACK(destroy_applet), ad);
	gtk_widget_show(ad->display);

	gtk_container_add(GTK_CONTAINER(ad->applet), ad->display);

	g_signal_connect(G_OBJECT(ad->applet),"change_orient",
			 G_CALLBACK(applet_change_orient), ad);

	g_signal_connect(G_OBJECT(ad->applet),"change_size",
			 G_CALLBACK(applet_change_size), ad);

	g_signal_connect(G_OBJECT(ad->applet),"change_background",
			 G_CALLBACK(applet_do_draw), ad);

	panel_applet_setup_menu_from_file(applet, NULL, "GNOME_DividerApplet.xml",
					  NULL, applet_menu_verbs, ad);
	
	gtk_widget_show(GTK_WIDGET(ad->applet));

	style_reload(ad);

	return TRUE;
}

static gboolean applet_factory(PanelApplet *applet, const gchar *iid, gpointer data)
{
	gboolean retval = FALSE;

	if (strcmp(iid, "OAFIID:GNOME_DividerApplet") == 0)
		{
		retval = applet_widget_new(applet);
		}

	return retval;
}

PANEL_APPLET_BONOBO_FACTORY("OAFIID:GNOME_DividerApplet_Factory", PANEL_TYPE_APPLET,
			    "Divider-Applet", "0", applet_factory, NULL)

