/*
 * simple divider applet
 * (C) 2003 John Ellis
 *
 * Author: John Ellis
 *
 * This software is released under the GNU General Public License (GNU GPL).
 * Please read the included file COPYING for more information.
 * This software comes with no warranty of any kind, use at your own risk!
 *
 */

#include "divider.h"

#include <panel-applet-gconf.h>
#include <gconf/gconf.h>
#include <gconf/gconf-client.h>


void property_load(AppData *ad)
{
	GConfClient *client;
	gchar *key;
	GError *error = NULL;
	
	g_free(ad->style_filename);
	ad->style_filename = NULL;

	client = gconf_client_get_default();
	key = panel_applet_get_preferences_key(ad->applet);

	if (gconf_client_dir_exists(client, key, NULL))
		{
		ad->visibility = panel_applet_gconf_get_int(ad->applet, "visibility", &error);
		if (error)
			{
			g_print ("%s \n", error->message);
			g_error_free (error);
			error = NULL;
			}
		ad->style_filename = panel_applet_gconf_get_string(ad->applet, "theme", NULL);

		ad->shift = panel_applet_gconf_get_bool(ad->applet, "shift_enabled", NULL);

		ad->shift_r = panel_applet_gconf_get_int(ad->applet, "shift_red", NULL);
		ad->shift_g = panel_applet_gconf_get_int(ad->applet, "shift_green", NULL);
		ad->shift_b = panel_applet_gconf_get_int(ad->applet, "shift_blue", NULL);
		ad->shift_a = panel_applet_gconf_get_int(ad->applet, "shift_alpha", NULL);
		}

	g_object_unref(G_OBJECT(client));
	g_free(key);
}

void property_save(AppData *ad)
{
	/* no op */
}

static void visibility_cb(GtkObject *adj, gpointer data)
{
	AppData *ad = data;

	ad->visibility = (gint)(GTK_ADJUSTMENT(adj)->value / 100.0 * 255.0);
	panel_applet_gconf_set_int(PANEL_APPLET(ad->applet), "visibility", ad->visibility, NULL);

	style_backing_sync(ad);
}

static void shift_enable_cb(GtkWidget *widget, gpointer data)
{
	AppData *ad = data;

	ad->shift = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	panel_applet_gconf_set_bool(PANEL_APPLET(ad->applet), "shift_enable", ad->shift, NULL);

	style_backing_sync(ad);
}

static void shift_color_cb(GnomeColorPicker *cp, guint r, guint g, guint b, guint a, gpointer data)
{
	AppData *ad = data;

	ad->shift_r = r >> 8;
	ad->shift_g = g >> 8;
	ad->shift_b = b >> 8;

	panel_applet_gconf_set_int(PANEL_APPLET(ad->applet), "shift_red", ad->shift_r, NULL);
	panel_applet_gconf_set_int(PANEL_APPLET(ad->applet), "shift_green", ad->shift_g, NULL);
	panel_applet_gconf_set_int(PANEL_APPLET(ad->applet), "shift_blue", ad->shift_b, NULL);

	if (ad->shift) style_backing_sync(ad);
}

static void shift_alpha_cb(GtkObject *adj, gpointer data)
{
	AppData *ad = data;

	ad->shift_a = (gint)(GTK_ADJUSTMENT(adj)->value / 100.0 * 255.0);
	panel_applet_gconf_set_int(PANEL_APPLET(ad->applet), "shift_alpha", ad->shift_a, NULL);

	if (ad->shift) style_backing_sync(ad);
}

static void style_select_cb(GtkTreeSelection *selection, gpointer data)
{
	AppData *ad = data;
	GtkTreeView *treeview;
	GtkTreeModel *model;
	GtkTreeIter iter;
	gchar *name;

	if (!gtk_tree_selection_get_selected (selection, NULL, &iter)) return;
	treeview = gtk_tree_selection_get_tree_view(selection);
	model = gtk_tree_view_get_model(treeview);
	gtk_tree_model_get(model, &iter, 0, &name, -1);
	if (name)
		{
		panel_applet_gconf_set_string(PANEL_APPLET(ad->applet), "theme", name, NULL);

		style_set(ad, name);

		g_free(name);
		}
}

static gint sort_style_list_cb(void *a, void *b)
{
	return strcmp((gchar *)a, (gchar *)b);
}

static void populate_theme_list(GtkListStore *liststore, GtkTreeView *listview, AppData *ad)
{

	DIR *dp;
	struct dirent *dir;
	struct stat ent_sbuf;
	GList *style_list = NULL;
	GList *list;

	if((dp = opendir(DIVIDER_THEME_PATH))==NULL)
		{
		/* dir not found */
		return;
		}

	while ((dir = readdir(dp)) != NULL)
		{
		/* skips removed files */
		if (dir->d_ino > 0)
			{
			gchar *name;
			gchar *path;

			name = dir->d_name; 
			path = g_concat_dir_and_file(DIVIDER_THEME_PATH, name);

			if (stat(path,&ent_sbuf) >= 0 && S_ISREG(ent_sbuf.st_mode))
				{
				style_list = g_list_insert_sorted(style_list,
						g_strdup(name),
						(GCompareFunc) sort_style_list_cb);
				}
			g_free(path);
			}
		}
        closedir(dp);

	list = style_list;
	while (list)
		{
		GtkTreeIter iter;

		gtk_list_store_append(liststore, &iter);
		gtk_list_store_set(liststore, &iter,
			           0, list->data,
				   -1);

		if ((ad->style_filename && strcmp(ad->style_filename, list->data) == 0) ||
		    (!ad->style_filename && strcmp(DEFAULT_STYLE, list->data) == 0) )
			{
			GtkTreeSelection *selection;

			selection = gtk_tree_view_get_selection(listview);
			gtk_tree_selection_select_iter(selection, &iter);
			}

		g_free(list->data);
		list = list->next;
		}

	g_list_free(style_list);
}

static gint property_destroy_cb(GtkWidget *widget, gpointer data)
{
	AppData *ad = data;
	ad->propwindow = NULL;

	return FALSE;
}

static void property_response_cb(GtkWidget *widget, gpointer data)
{
	gtk_widget_destroy(widget);
}

void property_show(PanelApplet *applet, gpointer data)
{
	AppData *ad = data;
	GtkWidget *notebook;
	GtkWidget *frame;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *vbox2;
	GtkWidget *label;
	GtkWidget *scrolled;
	GtkListStore *liststore;
	GtkWidget *listview;
	GtkTreeViewColumn *column;
	GtkTreeSelection *selection;
	GtkObject *adj;
	GtkWidget *hscale;
	GtkWidget *button;
	GtkWidget *color_picker;

	if(ad->propwindow)
		{
                gdk_window_raise(ad->propwindow->window);
                return;
		}

	ad->propwindow = gtk_dialog_new_with_buttons(_("Divider Preferences"), NULL,
						     GTK_DIALOG_DESTROY_WITH_PARENT,
						     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
						     NULL);

	notebook = gtk_notebook_new();
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(ad->propwindow)->vbox), notebook, TRUE, TRUE, 0);
	gtk_widget_show(notebook);
	
	vbox = gtk_vbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width(GTK_CONTAINER(vbox), GNOME_PAD_SMALL);

	scrolled = gtk_scrolled_window_new(NULL, NULL);
	gtk_widget_set_size_request(scrolled, -1, 200);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
				GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
	gtk_box_pack_start(GTK_BOX(vbox), scrolled, TRUE, TRUE, 0);
	gtk_widget_show(scrolled);

	/* style list */
	liststore = gtk_list_store_new(1, G_TYPE_STRING);
	listview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(liststore));
	g_object_unref(liststore);
	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(listview));
	gtk_tree_selection_set_mode(GTK_TREE_SELECTION(selection), GTK_SELECTION_SINGLE);

	column = gtk_tree_view_column_new_with_attributes(_("Image"), gtk_cell_renderer_text_new(),
							  "text", 0, NULL);
	gtk_tree_view_column_set_sizing(column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
	gtk_tree_view_append_column(GTK_TREE_VIEW(listview), column);

	gtk_container_add(GTK_CONTAINER(scrolled), listview);
	gtk_widget_show(listview);

	/* visibility */

	frame = gtk_frame_new(_("Visibility strength (%)"));
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

        adj = gtk_adjustment_new((float)ad->visibility / 255.0 * 100.0, 0.0, 100.0, 1, 5, 0);
	hscale = gtk_hscale_new(GTK_ADJUSTMENT(adj));
	gtk_range_set_update_policy(GTK_RANGE(hscale), GTK_UPDATE_DELAYED);
	gtk_scale_set_digits(GTK_SCALE(hscale), 0);
	g_signal_connect(G_OBJECT(adj), "value_changed",
		         G_CALLBACK(visibility_cb), ad);
	gtk_container_add(GTK_CONTAINER(frame), hscale);
	gtk_widget_show(hscale);

	frame = gtk_frame_new(_("Color"));
	gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 0);
	gtk_widget_show(frame);

	vbox2 = gtk_vbox_new(FALSE, 0);
	gtk_container_add(GTK_CONTAINER(frame), vbox2);
	gtk_widget_show(vbox2);

	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	button = gtk_check_button_new_with_label (_("Enable color"));
	gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), ad->shift);
	g_signal_connect(G_OBJECT(button), "clicked",
			 G_CALLBACK(shift_enable_cb), ad);
	gtk_widget_show(button);

	color_picker = gnome_color_picker_new();
	gnome_color_picker_set_title (GNOME_COLOR_PICKER(color_picker), _("Panel divider color"));
	gnome_color_picker_set_i8(GNOME_COLOR_PICKER(color_picker),
				 ad->shift_r, ad->shift_g, ad->shift_b, ad->shift_a);
	g_signal_connect(G_OBJECT(color_picker), "color_set",
			 G_CALLBACK(shift_color_cb), ad);

	gtk_box_pack_start(GTK_BOX(hbox), color_picker, FALSE, FALSE, 0);
	gtk_widget_show(color_picker);

	hbox = gtk_hbox_new(FALSE, GNOME_PAD_SMALL);
	gtk_container_set_border_width(GTK_CONTAINER(hbox), GNOME_PAD_SMALL);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
	gtk_widget_show(hbox);

	label = gtk_label_new (_("Strength (%):"));
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
	gtk_widget_show(label);

	adj = gtk_adjustment_new((float)ad->shift_a / 255.0 * 100.0, 0.0, 100.0, 1.0, 1.0, 0.0);
	hscale = gtk_hscale_new (GTK_ADJUSTMENT(adj));
	gtk_widget_set_size_request(hscale, 150, -1);
	gtk_range_set_update_policy (GTK_RANGE (hscale), GTK_UPDATE_DELAYED);
	gtk_scale_set_digits (GTK_SCALE (hscale), 0);
	g_signal_connect(G_OBJECT(adj), "value_changed",
			 G_CALLBACK(shift_alpha_cb), ad);
	gtk_box_pack_start(GTK_BOX(hbox), hscale, TRUE, TRUE, 0);
	gtk_widget_show(hscale);

	populate_theme_list(liststore, GTK_TREE_VIEW(listview), ad);
	g_signal_connect(selection, "changed",
			 G_CALLBACK(style_select_cb), ad);

        label = gtk_label_new(_("Style"));
        gtk_widget_show(vbox);
	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox, label);

	g_signal_connect(G_OBJECT(ad->propwindow), "response",
			 G_CALLBACK(property_response_cb), ad);
	g_signal_connect(G_OBJECT(ad->propwindow), "destroy",
			 G_CALLBACK(property_destroy_cb), ad);

	gtk_widget_show_all(ad->propwindow);
	return;
} 
